{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Password reset complete" %}{% endblock %}

{% block content %}
<p>
    {% trans "Your password has been reset!" %}
    {% blocktrans %}You may now <a href="{{ login_url }}">log in</a>{% endblocktrans %}.

</p>
{% endblock %}


{# This is used by django.contrib.auth #}
