{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Password reset" %}{% endblock %}

{% block content %}
<p>
    {% blocktrans %}
    We have sent you an email with a link to reset your password. Please check
    your email and click the link to continue.
    {% endblocktrans %}
</p>
{% endblock %}


{# This is used by django.contrib.auth #}
