{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Registration is closed" %}{% endblock %}

{% block content %}
<p>{% trans "Sorry, but registration is closed at this moment. Come back later." %}</p>
{% endblock %}
