{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Reset password" %}{% endblock %}

{% block content %}
<p>
    {% blocktrans %}
    Forgot your password? Enter your email in the form below and we'll send you instructions for creating a new one.
    {% endblocktrans %}
</p>
<form method="post" action="">
    {% csrf_token %}
    {{ form.as_p }}
    <input type="submit" value="{% trans 'Reset password' %}" />
</form>
{% endblock %}


{# This is used by django.contrib.auth #}
